/* eslint-disable @typescript-eslint/no-explicit-any */
import { throwError } from 'rxjs';
import { concatMap } from 'rxjs/operators';
import { initialRxRequestInit } from './config';
import { _fetch } from './request';
import { handleResponseError, parseResponseType } from './response';
import { parseInitOpts, splitInitArgs } from './util';
/**
 * Observable fetch
 *
 * parameter init ignored during parameter input is typeof Request
 */
export function rxfetch(input, init) {
    /* istanbul ignore else */
    if (!input) {
        throwError(() => new TypeError('value of input invalid'));
    }
    const initOpts = init ? Object.assign(Object.assign({}, initialRxRequestInit), init) : Object.assign({}, initialRxRequestInit);
    const options = splitInitArgs(initOpts);
    const { args, requestInit } = parseInitOpts(options);
    const dataType = args.dataType;
    const req$ = _fetch(input, args, requestInit);
    const ret$ = req$.pipe(concatMap(res => handleResponseError(res, dataType === 'bare')), concatMap(res => parseResponseType(res, dataType)));
    return ret$;
}
/** Observable GET method of fetch() */
export function get(input, init) {
    const ps = init
        ? Object.assign(Object.assign({}, init), { method: 'GET' }) : { method: 'GET' };
    return rxfetch(input, ps);
}
/** Observable POST method of fetch() */
export function post(input, init) {
    const ps = init
        ? Object.assign(Object.assign({}, init), { method: 'POST' }) : { method: 'POST' };
    return rxfetch(input, ps);
}
/** Observable PUT method of fetch() */
export function put(input, init) {
    const ps = init
        ? Object.assign(Object.assign({}, init), { method: 'PUT' }) : { method: 'PUT' };
    return rxfetch(input, ps);
}
/** Observable DELETE method of fetch() */
export function remove(input, init) {
    const ps = init
        ? Object.assign(Object.assign({}, init), { method: 'DELETE' }) : { method: 'DELETE' };
    return rxfetch(input, ps);
}
/** Update initialRxRequestInit */
export function setGloalRequestInit(config) {
    for (const [key, value] of Object.entries(config)) {
        Object.defineProperty(initialRxRequestInit, key, {
            configurable: true,
            enumerable: true,
            writable: true,
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            value,
        });
    }
}
/** Get copy of initialRxRequestInit */
export function getGloalRequestInit() {
    return Object.assign({}, initialRxRequestInit);
}
